#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <pthread.h>

#include <pv.h>

static char *prompt="test";
static int shutdown = 0;
static const int jv_port = 2202; 

void signalhandler (int sig)
{
	if (sig == SIGPIPE) return;
	if (sig == SIGHUP) {
		fprintf(stderr, "%s SIGHUP received\n", prompt);
	}
	if (sig == SIGTERM || sig == SIGINT) {
		fprintf(stderr, "%s received SIGTERM or SIGINT, shutting down\n", prompt);
		shutdown = 1;
	}
}

int main(int argc, char *argv[])
{
	struct sigaction action;
	pv_engine_t *pv_engine;

	unsigned char *pv_ucResetCounter;
	unsigned char *pv_choke_me;

	pv_engine = pv_init(argc, argv);
	if (pv_engine == NULL) 
		return EXIT_FAILURE;

	printf("%s: engine created\n", prompt);

	/* setup signal handler */
	action.sa_handler = signalhandler;
	sigemptyset (&action.sa_mask);
	action.sa_flags = 0;
	sigaction (SIGTERM, &action, 0);	/* catches kill from the shell   */
	sigaction (SIGINT,  &action, 0);	/* catches Ctrl-C from the shell */
	sigaction (SIGPIPE, &action, 0);	/* catches Broken Pipe exception */
	sigaction (SIGHUP,  &action, 0);	/* catches HUP                   */

	pv_ucResetCounter = pv_register_uint8(pv_engine, "ucResetCounter");
	pv_choke_me = pv_register_uint8(pv_engine, "pv_choke_me");
	printf("%s: map=%p\n", prompt, pv_engine->pv_map);
	printf("%s: pv_ucResetCounter=%p\n", prompt, pv_ucResetCounter);
	printf("%s: pv_choke_me=%p\n", prompt, pv_choke_me);


	while (!shutdown) {

		pv_lock(pv_engine);
		(*pv_ucResetCounter)++;
		pv_flush(pv_engine, pv_ucResetCounter, sizeof(*pv_ucResetCounter));

		pv_unlock(pv_engine);
		
		printf("%s: ucResetCounter=%i\n", prompt, *pv_ucResetCounter);

		sleep(1);
		
	}
	
	pv_finish(pv_engine);
	
	return EXIT_SUCCESS;
}

